@@ -315,7 +315,14 @@ gedit_file_save (GeditMDIChild* child)
 		view = GTK_WIDGET (g_list_nth_data (
 					bonobo_mdi_child_get_views (BONOBO_MDI_CHILD (child)), 0));
 		if (view != NULL)
+		{
+			GtkWindow *window;
+
+			window = GTK_WINDOW (bonobo_mdi_get_window_from_view (view));
+			gtk_window_present (window);
+
 			bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
+		}
 		
 		gedit_utils_error_reporting_saving_file (uri, error,
 					GTK_WINDOW (gedit_get_active_window ()));
@@ -372,11 +379,17 @@ gedit_file_save_as (GeditMDIChild *child)
 
 	if (gedit_document_is_untitled (doc))
 	{
+		char *tmpstr;
+		
 		path = (gedit_default_path != NULL) ? 
 			g_strdup (gedit_default_path) : NULL;
 
-		/* FIXME: check the case in which "Untitled n" is a non-ascii string */
-		fname = gedit_document_get_uri (doc);		
+		tmpstr = gedit_document_get_uri (doc);
+		fname = g_filename_from_utf8 (tmpstr, -1, NULL, NULL, NULL);
+		if (!fname)
+			/* FIXME: should fname remain NULL in this case? - Paolo */		
+			fname = g_strdup ("Untitled"); /* Use ASCII */
+		g_free (tmpstr);
 	}
 	else
 	{
@@ -582,7 +595,14 @@ gedit_file_save_all (void)
 	}
 
 	if (view !=  bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)))
+	{
+		GtkWindow *window;
+
+		window = GTK_WINDOW (bonobo_mdi_get_window_from_view (view));
+		gtk_window_present (window);
+
 		bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
+	}
 }
 
 gboolean
@@ -677,24 +697,21 @@ gedit_file_open_uri_list (GList* uri_list, gint line, gboolean create)
 				if (create)
 				{
 					GtkWidget *dialog;
-					gchar *str;
 					gboolean created = FALSE;
 					gchar *formatted_path;
 
 					formatted_path = eel_format_uri_for_display (full_path);
 					g_return_val_if_fail (formatted_path != NULL, FALSE);
 					
-					str = g_strdup_printf (_("The file \"%s\" does not exist. Would you like to create it?"),
-							       formatted_path);
-		
-					g_free (formatted_path);
-
 					dialog = gtk_message_dialog_new (
 						GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
 						GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 					   	GTK_MESSAGE_QUESTION,
 					   	GTK_BUTTONS_YES_NO,
-						str);
+						_("The file \"%s\" does not exist. Would you like to create it?"),
+						formatted_path);
+
+					g_free (formatted_path);
 
 					gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_NO);
 
@@ -710,8 +727,6 @@ gedit_file_open_uri_list (GList* uri_list, gint line, gboolean create)
 									
 					gtk_widget_destroy (dialog);
 
-					g_free (str);
-					
 					if (created)
 					{
 						if (gedit_file_open_real (full_path, 
@@ -734,11 +749,16 @@ gedit_file_open_uri_list (GList* uri_list, gint line, gboolean create)
 	if (view != NULL)
 	{
 		GeditDocument *doc;
+		GtkWindow *window;
 
 		doc = gedit_view_get_document (view);
 		g_return_val_if_fail (doc, FALSE);
 		
 		gedit_document_goto_line (doc, MAX (0, line - 1));
+
+		window = GTK_WINDOW (bonobo_mdi_get_window_from_view (GTK_WIDGET (view)));
+		gtk_window_present (window);
+
 		bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), GTK_WIDGET (view));		
 	}
 
